import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects

def permutation_crystal_multi(indiv, Optimizer):
    """Move function to perform Permutation of multiple atom based on atomlist with entire crystal in Defect
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure != 'Defect':
        Optimizer.output.write('WARNING: Crystal extended permutation attempted on non-Defect system. Skipping Mutation\n')
    else:
        indexbulk=len(indiv[0])
        solid=Atoms()
        solid.extend(indiv[0])
        solid.extend(indiv.bulki)
        solid.set_pbc(True)
        solid.set_cell(indiv[0].get_cell())
        a1=solid[random.randint(0,indiv[0].get_number_of_atoms()-1)]
        opts=[inds for inds in solid if inds.symbol != a1.symbol]
        inpts=[inds for inds in solid if inds.symbol == a1.symbol]
        maxperm=len(inpts)
        if maxperm > len(opts):
            maxperm=len(opts)
        Optimizer.output.write('Multiatom Crystal-wide Permutation Mutation performed on individual\n')
        Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
        if maxperm==0:
            Optimizer.output.write('WARNING: Permutation Mutation attempted on single atom structure system\n')
            natperm=0
        else:
            natperm=random.randint(1,maxperm/2)
        Optimizer.output.write('Number of atoms permutated = '+repr(natperm)+'\n')
        for i in range(natperm):
            a1=inpts[random.randint(0,len(inpts)-1)]
            a2=opts[random.randint(0,len(opts)-1)]
            solid[a1.index].symbol, solid[a2.index].symbol=a2.symbol, a1.symbol
            Optimizer.output.write(repr(a1.symbol) + ' swapped with ' + repr(a2.symbol)+'\n')
        indiv.bulki=solid[indexbulk::]
        indiv[0]=solid[0:indexbulk]
        Optimizer.output.write(repr(indiv[0])+'\n')
        for sym,c,m,u in Optimizer.atomlist:
            nc=len([atm for atm in solid if atm.symbol==sym])
            Optimizer.output.write('Defect configuration contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        muttype='PCMA'+repr(natperm)
        if indiv.energy==0:
            indiv.history_index=indiv.history_index+'m'+muttype
        else:
            indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


import random

def fuss(pop,nkeep, Optimizer):
	"""Selection function to employ basic Fixed Uniform Selection Scheme to select and 
	order individuals in a population.
	Picks one fuss point for the selection process
	fuss point must reside within fusslimit of minimum energy structure
	"""
	fusslimit = Optimizer.fusslimit
	#picks one fuss point
	newpop = []
	# Collect fitnesses
	fits = [ind.fitness for ind in pop]
	minindex = min(xrange(len(fits)), key=fits.__getitem__)
	#Find min and max fitness
	minf = min(fits)
	maxf = max(fits)
	if abs(maxf-minf) > fusslimit:
		maxf = minf + fusslimit
	#Select random point on fitness line
	pt = random.uniform(minf,maxf)
	#Calculate the distance of each individual's fitness from that point
	i = 0
	distances = []
	for fit in fits:
		distances.append((abs(pt-fit),i))
		i += 1
	#Sort distances from min to max
	dlist = sorted(distances, key=lambda one: one[0])
	#Select individuals with lowest distance
	indices = []
	for i in range(nkeep):
		newpop.append(pop[dlist[i][1]])
		indices.append(dlist[i][1])
	#Always keep lowest energy individual
	if minindex not in indices:
		rm = random.choice(indices)
		newpop = [pop[inx] for inx in indices if inx != rm]
		newpop.append(pop[minindex])
	
	return newpop


import random
from MAST.structopt.tools.find_defects import find_defects

def zp_rotation(indiv, Optimizer):
    """Move function to perform Zero point rotation of atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    ax=['x', '-x','y','-y','z','-z']
    #Identify random rotation about axis and rotate all atoms
    rax=ax[random.randint(0,len(ax)-1)]
    rang=random.uniform(30,180)
    #rang=random.random()*90
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            atms,indb,vacant,swap,stro = find_defects(indiv[0],Optimizer.solidbulk,0)
        else:
            atms = indiv[0]
    else:
        atms=indiv[0]
    atms.rotate(rax,a=rang,center='COM',rotate_cell=False)
    if Optimizer.structure=='Defect':
        if Optimizer.isolate_mutation:
            atms.extend(indb)
    indiv[0]=atms.copy()
    Optimizer.output.write('Zero point full rotation performed on individual\n')
    Optimizer.output.write('Index = '+repr(indiv.index)+'\n')
    Optimizer.output.write('Rotation vector = '+repr(rax)+'\n')
    Optimizer.output.write('Rotation angle = '+repr(rang)+'\n')
    muttype='ZPR'
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv


from MAST.structopt.tools import eval_energy
from MAST.structopt.inp_out.write_xyz import write_xyz
import logging
import math
import os

def stem_cost_rotation(indiv, Optimizer):
    '''Function to calculate STEM_Cost fitness of individual.
    Input:
        indiv = structopt Individual class object to be evaluated
        Optimizer = structopt Optimizer class object
            Must have STEM Calc function attached
    Output:
        indiv = structopt Individual class object with new fitness.
    '''
    if 'FIT' in Optimizer.debug:
        debug=True
    else:
        debug=False
    logger = logging.getLogger(Optimizer.loggername)
    starting = indiv.duplicate()
    cwd = os.getcwd()
    try:
        outs = eval_energy(Optimizer,indiv)
    except Exception, e:
        logger.warn('Error in energy evaluation: {0}'.format(e), exc_info=True)
        stro = 'ERROR: Problem in Energy Evaluation'
        print stro
        print e
        stro += '\n' + repr(e)
        os.chdir(cwd)
        f=open('problem-structures.xyz','a')
        totalsol = indiv[0].copy()
        totalsol.extend(indiv.bulki)
        write_xyz(f,totalsol,data='Starting structure hindex={0}'.format(indiv.history_index))
        indiv.energy = 10000
        f.close()
        print '    Writing structure to problemstructures.xyz file. Structure (hindex) : '+indiv.history_index
        print '    Setting individual energy to 50000.'
        outs = [10000, starting.bulki, starting, stro]
    indiv.energy = outs[0]
    stro=outs[3]
    if Optimizer.structure == 'Defect' or Optimizer.structure=='Surface':
        indiv.bulki = outs[1]
    fit = indiv.energy
    if abs(fit) > Optimizer.energy_cutoff_factor*(len(indiv[0])+len(indiv.bulki)):
        message = 'Warning: Found oddly large energy from Lammps in structure HI={0}'.format(indiv.history_index)
        logger.warn(message)
        print message
        print '    Setting fitness to 10000'
    if math.isnan(fit):
        logger.warn('Found NAN energy structure HI={0}'.format(indiv.history_index))
        indiv.energy = 10000
    
    #Calculate the chisq
    chisq0 = Optimizer.stemcalc.run(indiv[0])
    #Identify the 12 <110> directions
    indatoms = indiv[0].copy()
    zdir_list = get_z_directions(indatoms)
    chisqlist = []
    for direction in zdir_list:
        #align atoms so z direction is is in direction
        indatoms.rotate_euler(direction[0], direction[1], direction[2])
        chisqn, phi = find_optimial_phi_degree(atoms, debug=debug)
        chisqlist.append([chisqn,phi,direction])
    minchi = min([chi for chi, phi,direction in chisqlist])
    minphi = [one[1] for one in chisqlist if one[0]==minchi][0]
    minz = [one[2] for one in chisqlist if one[0]==minchi][0]
    atomsnew = indiv[0].copy().rotate_euler(minz[0],minz[1],minz[2])
    atomsnew.rotate(minphi)
    indiv[0]=atomsnew.copy()
    #Calculate the fitness of the individual
    indiv.fitness=indiv.energy+Optimizer.stem_coeff*minchi
    #Output values of fitness, energy, rms, and alpha for comparison
    stro+='Individual '+repr(indiv.history_index)+': Fitness '+repr(indiv.fitness)+', Energy '+repr(indiv.energy)+', Chi^2 '+repr(chisq)+', Alpha '+repr(Optimizer.stem_coeff)+'\n'
    return indiv, stro

def get_z_directions(atoms):
    """Funciton to calculate the z directions from an atoms structure
    Input:
        atoms = atoms structure
    Output:
        z_dirs = list of angles for euler rotation representing 12 z directions"""
    #Do function
    return z_dirs

def find_optimal_psi_degree(atoms, debug):
    chisq = Optimizer.stemcalc.run(indiv[0])
    if debug:
        print angle
    return chisq, atomsn



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os

import numpy as np
import pymatgen as pmg

from MAST.utility import InputOptions
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import MAST2Structure
from MAST.utility.mastfile import MASTFile

ALLOWED_KEYS = {\
                 'inputfile'    : (str, 'mast.inp', 'Input file name'),\
               }

class IndepLoopInputParser(MASTObj):

    """Scans an input file for "indeploop" keyword and copies it into
        many input files.
        Attributes:
            self.indeploop <str>: flag for independent looping
            self.loop_delim <str>: character for delimiting loops
            self.loop_start <str>: character indicating start of loop
            self.loop_end <str>: character indicating end of loop
            self.baseinput <MASTFile>: MASTFile created from *.inp input file
            self.pegloop1 <str>: flag for one pegged loop
            self.pegloop2 <str>: flag for second pegged loop

        Looping must be indicated at the beginning of the line, and the
        text to be looped must be complete:
        indeploop mast_kpoints (3x3x3 G, 5x5x5 G, 2x2x2 M, 4x4x4 M) 
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
        self.indeploop = "indeploop"
        self.loop_delim = ","
        self.loop_start = "("
        self.loop_end = ")"
        self.baseinput = MASTFile(self.keywords['inputfile'])
        self.pegloop1 = "pegloop1"
        self.pegloop2 = "pegloop2"
    
    def main(self, verbose=0):
        """Scan for independent loops and set up dictionaries.
            Return:
                createdfiles <list of str>: list of created input files
        """
        indepdict=self.scan_for_loop(self.indeploop)
        pegdict1 = self.scan_for_loop(self.pegloop1)
        pegdict2 = self.scan_for_loop(self.pegloop2)
        if len(indepdict.keys()) == 0 and len(pegdict1.keys()) == 0 and len(pegdict2.keys()) == 0:
            return dict()
        alldict = dict(indepdict)
        alldict.update(pegdict1)
        alldict.update(pegdict2)
        indepcomb=self.get_combo_list(indepdict, 0)
        pegcomb1=self.get_combo_list(pegdict1, 1)
        pegcomb2=self.get_combo_list(pegdict2, 1)
        allcombs = self.combine_three_combo_lists(indepcomb, pegcomb1, pegcomb2)
        datasets = self.prepare_looped_datasets(alldict, allcombs)
        createdfiles = self.create_input_files(datasets)
        if verbose == 1:
            self.print_list(indepcomb)
            self.print_list(pegcomb1)
            self.print_list(pegcomb2)
            self.print_list(allcombs)
            for datakey in datasets:
                self.print_list(datasets[datakey])
        return createdfiles

    def scan_for_loop(self, loopkey):
        """Scan for loops.
            Args:
                loopkey <str>: string for searching for loops. This is
                        either "indeploop" or "pegloop"
            Return:
                loopdict <dict>: Dictionary where loop line indices in the file
                                are keys, with values
                                'looplist', containing a list of strings, and
                                'prepend', containing any text to prepend
                                'append', containing any text to append
                    e.g. loopdict[12]['looplist']=['3x3x3 G','5x5x5 G']
                                     ['prepend']='mast_kpoints '
                                     ['append']=''
        """
        loopdict=dict()
        realline=""
        split1=""
        split2=""
        numlines = len(self.baseinput.data)
        lidx =0
        while lidx < numlines:
            dataline = self.baseinput.data[lidx].strip()
            if loopkey in dataline:
                loopdict[lidx] = dict()
                realline=dataline.split(' ',1)[1] #strip off indeploop
                split1 = realline.split(self.loop_start)
                split2 = split1[1].split(self.loop_end)
                loopdict[lidx]['prepend'] = split1[0]
                loopdict[lidx]['append'] = split2[1]
                loopdict[lidx]['looplist'] = split2[0].split(self.loop_delim)
            lidx = lidx + 1
        #print "TTM DEBUG: ", loopdict
        if len(loopdict.keys()) == 0:
            return dict()
        return loopdict
    
    def get_combo_list(self, loopdict, pegged=0):
        """Prepare a combination list of looping indices.
            Args:
                loopdict <dict>: dictionary of looping items from
                                    scan_for_loop
            Returns:
                combolist <list of str>: list of strings, like ["3-0","5-1"]
                            representing combinations
        """
        combolist=list()
        flatlists=list()
        loopkeys = list(loopdict.keys())
        loopkeys.sort()
        if pegged == 0:
            for loopkey in loopkeys:
                numloop = len(loopdict[loopkey]['looplist'])
                loopct=0
                flatlist=list()
                while loopct < numloop:
                    flatlist.append(str(loopkey) + '-' + str(loopct))
                    loopct = loopct + 1
                flatlists.append(flatlist)
            import itertools
            prod_list = itertools.product(*flatlists)
            stopiter = 0
            while not stopiter:
                try:
                    mycomb = prod_list.next()
                except StopIteration:
                    stopiter = 1
                if stopiter == 0:
                    combolist.append(list(mycomb))
        elif pegged == 1:
            if len(loopkeys) == 0:
                return combolist #Empty list
            numloop = len(loopdict[loopkeys[0]]['looplist']) #all same len
            numct=0
            while numct < numloop:
                flatlist=list()
                for loopkey in loopkeys:
                    flatlist.append(str(loopkey) + '-' + str(numct))
                numct = numct + 1
                combolist.append(flatlist)
        #print "TTM DEBUG: ", flatlists
        return combolist

    def combine_three_combo_lists(self, indeplist, peglist1, peglist2):
        """Combine two pegged lists and one independent list.
            Args:
                indeplist <list of list>: List of indeploop combinations.
                peglist1 <list of list>: List of pegged loop 1 combinations
                peglist2 <list of list>: List of pegged loop 2 combinations
            Returns:
                alllist <list of list>: List of all combinations
        """
        templist=list()
        threelist=list()
        templist = self.combine_combo_lists(indeplist, peglist1)
        threelist = self.combine_combo_lists(templist, peglist2)
        return threelist

    def combine_combo_lists(self, indeplist, peggedlist):
        """Combine combination lists.
            Args:
                indeplist <list of list>: List of indeploop combinations.
                peggedlist <list of list>: List of pegged loop combinations
            Returns:
                alllist <list of list>: List of all combinations
        """
        alllist=list()
        if len(peggedlist) == 0:
            return indeplist
        if len(indeplist) == 0:
            return peggedlist
        for pegitem in peggedlist:
            for indepitem in indeplist:
                alllistentry = list(pegitem)
                alllistentry.extend(indepitem)
                alllist.append(alllistentry)
        return alllist



    def print_list(self, mylist):
        """Print a list.
            Args:
                mylist <list of str>
        """
        for myitem in mylist:
            print myitem
        return


    def prepare_looped_lines(self, alldict, comblist):
        """Prepare looped lines from looping dictionary.
            Args:
                loopdict <dict>: dictionary of looping items from
                                    scan_for_loop
            Returns:
                loopline_dict <dict of dict>: dictionary of different lines for
                                        input files, with keys being
                                        the looped line index
                
        """
        loopline_dict=dict()
        for stridx in comblist:
            lidx = int(stridx.split('-')[0])
            loopidx = int(stridx.split('-')[1])
            loopline_dict[lidx] =  alldict[lidx]['prepend'] + alldict[lidx]['looplist'][loopidx].strip() + alldict[lidx]['append'] + '\n'
        return loopline_dict

    def prepare_looped_datasets(self, alldict, allcombs):
        """Prepare looped datasets from looping lines.
            Args:
                alldict <dict>: line dictionary for looping
                allcombs <list of list>: index combinations for looping
            Returns:
                datasets_dict <dict of list>: full datasets for new input files
        """
        datasets_dict=dict()
        numcombs = len(allcombs)
        combct = 0
        while combct < numcombs:
            newdata = list(self.baseinput.data)
            loopedlines = dict()
            loopedlines = self.prepare_looped_lines(alldict, allcombs[combct])
            for lvalidx in loopedlines.keys():
                newdata[lvalidx] = loopedlines[lvalidx]
            datasets_dict[combct] = newdata
            combct = combct + 1
        return datasets_dict

    def create_input_files(self, datasets_dict):
        """Create independently looped input files.
            Args:
                datasets_dict <dict of list>

            Returns:
                createdfiles <list of str>: list of newly-created file names
            Creates an input file for each entry.


        """
        ifname = self.keywords['inputfile']
        dirstem = os.path.dirname(ifname)
        basename = os.path.basename(ifname).split('.')[0]
        createdfiles=list()
        if dirstem == "":
            dirstem = os.getcwd()
        dkeys = datasets_dict.keys()
        dkeys.sort()
        dct=1
        for didx in dkeys:
            newfile = MASTFile()
            newfile.data = list(datasets_dict[didx])
            newname="%s/loop_%s_%s.inp" % (dirstem, basename, str(dct).zfill(2))
            newfile.to_file(newname)
            createdfiles.append(os.path.basename(newname))
            dct=dct+1
        return createdfiles



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import time
import shutil
import pymatgen
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import fileutil
from MAST.utility import Metadata
from MAST.utility import MASTFile
from MAST.utility import loggerutils
from MAST.submit import queue_commands
from MAST.submit import script_commands
from pymatgen.core.structure import Structure
from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser
from custodian.custodian import ErrorHandler
try:
   from custodian.custodian import backup
except:
   from custodian.utils import backup
import logging

class VaspReachedNSWErrorHandler(ErrorHandler):
    """Check if a VASP job has reached NSW.
        Typically it should converge long before NSW.
        If NSW = 0 , ignore this error.
    """
    def __init__(self, ingpath, archivelist=list(), copyfromlist=list(), copytolist=list()):
        """
            Args: (archivelist is different from the custodian version)
                ingpath <str>: ingredient path
                archivelist <list of str>: list of file names to archive
                copyfromlist <list of str>: list of file names to copy from
                copytolist <list of str>: list of file names to copy to; one-to
                        -one correspondence with copyfromlist (e.g. copy from 
                        CONTCAR to POSCAR for VASP)
            Returns:
                Archives files to error.#.tar.gz
        """
        self.ingpath = ingpath
        self.archivelist = list(archivelist)
        self.copyfromlist = list(copyfromlist)
        self.copytolist = list(copytolist)
        self.logger = logging.getLogger(self.ingpath)
        self.logger = loggerutils.add_handler_for_recipe(self.ingpath, self.logger)
    
    def check(self):
        ifile = "%s/INCAR" % self.ingpath
        if os.path.isfile(ifile):
            myincar = pymatgen.io.vaspio.Incar.from_file(ifile)
        else:
            self.logger.warning("No INCAR file found at %s" % ifile)
            return False
        nsw = myincar.get('NSW',0)
        if nsw == 0:
            self.logger.info("NSW is 0 or not set.")
            return False
        ofile = "%s/OUTCAR" % self.ingpath
        if not (os.path.isfile(ofile)):
            ofile = "%s/01/OUTCAR" % self.ingpath
        if not (os.path.isfile(ofile)):
            self.logger.error("No OUTCAR file at %s or %s/01." % self.ingpath)
            return False
        myiters = fileutil.grepme(ofile, "Iter")
        if len(myiters) == 0:
            self.logger.warning("No Iterations in OUTCAR at %s" % ofile)
            return False
        lastentry = myiters[-1]
        ionichalf = lastentry.split('(')[0]
        lastiter = ionichalf.split()[-1]
        lastiter = int(lastiter)
        if lastiter >= nsw:
            self.logger.info("Last ionic step %s is at NSW %s" % (lastiter, nsw))
            return True
        return False
        #ofile = "%s/vasprun.xml" % self.ingpath
        #if os.path.isfile(ofile):
        #    myvasprun = pymatgen.io.vaspio.Vasprun(ofile)
        #    if myvasprun.nionic_steps >= nsw:
        #        self.logger.info("Ionic steps %s is at NSW %s" % (myvasprun.nionic_steps, nsw))
        #        return True
        #else:
        #    self.logger.error("No vasprun.xml file found at %s/vasprun.xml" % self.ingpath)
        #    return False

    def correct(self): 
        actions=list()
        backup(self.archivelist)
        actions.append("Archived files %s" % self.archivelist)
        for myfile in self.archivelist:
            if (not myfile in self.copyfromlist) and (not myfile in self.copytolist): #if it is a copy-from, don't want to delete it, and if it is a copy-to, it will be either overwritted by a copy-from file, or preserved
                if os.path.isfile(myfile):
                    os.remove(myfile)
        for myfile in self.copyfromlist:
            cindex = self.copyfromlist.index(myfile)
            if os.path.isfile(myfile):
                if os.stat(myfile).st_size > 0:
                    os.rename(myfile, self.copytolist[cindex])
                    actions.append("Copied file %s to %s" % (self.copyfromlist[cindex], self.copytolist[cindex]))
                else:
                    actions.append("Skipped file copy of %s to %s because %s was empty." % (self.copyfromlist[cindex],self.copytolist[cindex],self.copyfromlist[cindex]))
            else:
                actions.append("Skipped file copy of %s to %s because %s did not exist." % (self.copyfromlist[cindex],self.copytolist[cindex],self.copyfromlist[cindex]))
        return {"errors": ["MAST VASP at NSW error"], "actions": actions}

    @property
    def is_monitor(self): return True

    @property
    def to_dict(self): return {"@module": self.__class__.__module__, "@class": self.__class__.__name__, "output_filename": self.output_filename, "timeout": self.timeout}



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Glen Jenness
# Last updated: 2013-07-01
##############################################################
import os

from MAST.utility import MASTObj
from MAST.utility import MASTError

ALLOWED_KEYS = {
                'metafile' : (str, 'metadata.txt', 'Metadata file name')
               }


class Metadata(MASTObj):
    """Class to handle the metadata file
        Fields are composed of a keyword and a value, separated by a '='.
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)

    def write_data(self, keyword, data):
        """Writes a keyword and its associated data to the metafile"""
        with open(self.keywords['metafile'], 'a') as metafile:
            # First check to see if the keyword already exists in the metadata file
            if None in self.search_data(keyword):
                metafile.write('%s = %s\n' % (keyword, data))
            else:
                entry = self.read_data(keyword)
                entry += ', %s' % data
                self.clear_data(keyword)
                metafile.write('%s = %s\n' % (keyword, entry))

    def search_data(self, keyword):
        """Searches the file for a keyword, and if found returns the line number
            and data for that keyword.
            Returns:
                [line_number, data]
                line_number <int>: line number found
                data <str>: data found
        """
        line_number = None
        data = None

        with open(self.keywords['metafile'], 'r') as metafile:
            for n, line in enumerate(metafile):
                if (keyword.lower() + ' = ') in line.lower():
                    line_number = n
                    data = line.split(' = ')[1].strip()
                    break

        return line_number, data

    def read_data(self, keyword):
        """Searches the metadata file for a specific keyword and returns the
            data.
        """
        line_number, data = self.search_data(keyword)
        return data
 
    def clear_data(self, keyword):
        """Removes the specified data and keyword from the file.
            This is inefficient, as what we do is basically reading in the file
            then iterating through the file, do if..else check, clear the file,
            then re-write it.

            But it works!
        """
        line_number, data = self.search_data(keyword)

        data = list()
        with open(self.keywords['metafile'], 'r') as metafile:
            for n, line in enumerate(metafile):
                if n == line_number:
                    pass
                else:
                    data.append(line)

        self.clear_file()
        for line in data:
            keyword, data = line.split(' = ')
            self.write_data(keyword, data.strip())

    def clear_file(self):
        """Empties the metafile"""
        open(self.keywords['metafile'], 'w').close()

    def __repr__(self):
        return file(self.keywords['metafile']).read()



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import os
import time
import shutil
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import dirutil
from MAST.utility import Metadata
from MAST.utility import MASTFile
from MAST.utility import loggerutils
from MAST.submit import queue_commands
from MAST.submit import script_commands
from pymatgen.core.structure import Structure
from pymatgen.io.vaspio import Poscar
from pymatgen.io.cifio import CifParser
from custodian.custodian import ErrorHandler
try:
   from custodian.custodian import backup
except:
   from custodian.utils import backup
import logging

class MASTWalltimeErrorHandler(ErrorHandler):
    """Check if a job has exceeded its walltime."""
    def __init__(self, ingpath, archivelist=list(), copyfromlist=list(), copytolist=list()):
        """
            Args: (archivelist is different from the custodian version)
                ingpath <str>: ingredient path
                archivelist <list of str>: list of file names to archive
                copyfromlist <list of str>: list of file names to copy from
                copytolist <list of str>: list of file names to copy to; one-to
                        -one correspondence with copyfromlist (e.g. copy from 
                        CONTCAR to POSCAR for VASP)
            Returns:
                Archives files to error.#.tar.gz
        """
        self.ingpath = ingpath
        self.archivelist = list(archivelist)
        self.copyfromlist = list(copyfromlist)
        self.copytolist = list(copytolist)
        self.logger = logging.getLogger(self.ingpath)
        self.logger = loggerutils.add_handler_for_recipe(self.ingpath, self.logger)
    
    def check(self):
        errfilepath = queue_commands.get_job_error_file(self.ingpath)
        if errfilepath == None:
            return False
        errfile = MASTFile(errfilepath)
        for errline in errfile.data:
            if 'walltime' in errline.lower():
                return True
    
    def correct(self): 
        actions=list()
        backup(self.archivelist)
        actions.append("Archived files %s" % self.archivelist)
        for file in self.archivelist:
            if (not file in self.copyfromlist) and (not file in self.copytolist): #if it is a copy-from, don't want to delete it, and if it is a copy-to, it will be either overwritted by a copy-from file, or preserved
                if os.path.isfile(file):
                    os.remove(file)
        for file in self.copyfromlist:
            cindex = self.copyfromlist.index(file)
            if os.path.isfile(file):
                if os.stat(file).st_size > 0:
                    os.rename(file, self.copytolist[cindex])
                    actions.append("Copied file %s to %s" % (self.copyfromlist[cindex], self.copytolist[cindex]))
                else:
                    actions.append("Skipped file copy of %s to %s because %s was empty." % (self.copyfromlist[cindex],self.copytolist[cindex],self.copyfromlist[cindex]))
            else:
                actions.append("Skipped file copy of %s to %s because %s did not exist." % (self.copyfromlist[cindex],self.copytolist[cindex],self.copyfromlist[cindex]))
        return {"errors": ["MAST exceeded walltime error"], "actions": actions}

    @property
    def is_monitor(self): return True

    @property
    def to_dict(self): return {"@module": self.__class__.__module__, "@class": self.__class__.__name__, "output_filename": self.output_filename, "timeout": self.timeout}

class MASTMemoryErrorHandler(ErrorHandler):
    """Check if a job has insufficient virtual memory.
        If found, increases number of nodes and processors; typically
        the virtual memory requested is already at a max (at least for
        CMG queues).
    """
    def __init__(self, ingpath, keywords, archivelist=list()):
        """
            Args:
                ingpath <str>: ingredient path
                keywords <dict>: keyword dictionary
                archivelist <list of str>: list of file names to be archived 
                        e.g. OUTCAR (optional)
            Returns:
                modifies submission script to add more nodes
        """
        self.ingpath = ingpath
        self.archivelist = archivelist
        self.logger = logging.getLogger(self.ingpath)
        self.logger = loggerutils.add_handler_for_recipe(self.ingpath, self.logger)
        self.keywords = keywords

    def check(self):
        errfilepath = queue_commands.get_job_error_file(self.ingpath)
        self.logger.info("Checking file at %s" % errfilepath)
        if errfilepath == None:
            return False
        errfile = MASTFile(errfilepath)
        for errline in errfile.data:
            if 'insufficient virtual memory' in errline.lower():
                return True
    
    def correct(self): 
        actions=list()
        multiplier = 4
        
        if 'mast_nodes' in self.keywords['program_keys'].keys():
            currnodes = self.keywords['program_keys']['mast_nodes']
            newnodes = int(currnodes) * multiplier
            self.keywords['program_keys']['mast_nodes'] = newnodes
            actions.append("Multiplied mast_nodes by %i to get %i" % (multiplier, newnodes))
        
        if 'mast_processors' in self.keywords['program_keys'].keys():
            currprocs = self.keywords['program_keys']['mast_processors']
            newprocs = int(currprocs) * multiplier
            self.keywords['program_keys']['mast_processors'] = newprocs
            actions.append("Multiplied mast_processors by %i to get %i" % (multiplier, newprocs))
        script_commands.write_submit_script(self.keywords)
        actions.append("Wrote new submission script.")
        #archive old files. But, for insufficient virtual memory, not worth
        # trying to copy files, for example, CONTCAR to POSCAR, as the run
        # probably did not actually run to begin with.
        backup(self.archivelist)
        actions.append("Archived files %s" % self.archivelist)
        for fname in self.archivelist:
            if os.path.isfile("%s/%s" % (self.keywords['name'],fname)):
                os.remove("%s/%s" % (self.keywords['name'],fname))

        return {"errors": ["MAST insufficient virtual memory error"], "actions": actions}

    @property
    def is_monitor(self): return True

    @property
    def to_dict(self): return {"@module": self.__class__.__module__, "@class": self.__class__.__name__, "output_filename": self.output_filename, "timeout": self.timeout}
